import axios from "@/js/libs/axios";
import type { AxiosResponse } from "axios";
import { AxiosError } from "axios";

export type FetchOptions = Partial<{
  isFetching: boolean;
}>;
export type SubmitOptions<R = any, E = any> = Partial<{
  onStart: () => void;
  onSuccess: (response: AxiosResponse<R>) => void;
  onError: (error: AxiosError<E>) => void;
}>;

export default function useFetch<D = any>(url: string, options?: FetchOptions) {
  const hasErrors = ref<boolean>(false);
  const isFetching = ref<boolean>(options?.isFetching ?? false);
  const isFirthFetch = ref<boolean>(true);

  const fetch = async <R = any>(method: "get" | "post" | "put" | "patch" | "delete", data?: D, options?: SubmitOptions<R>): Promise<AxiosResponse<R>> => {
    const onStart = () => {
      isFetching.value = true;
      if (options?.onStart) {
        options.onStart();
      }
    };
    const onSuccess = (response: AxiosResponse<R>) => {
      hasErrors.value = false;
      isFetching.value = false;
      isFirthFetch.value = false;
      if (options?.onSuccess) {
        options.onSuccess(response);
      }
    };
    const onError = (error: AxiosError) => {
      hasErrors.value = true;
      isFetching.value = false;
      isFirthFetch.value = false;
      if (options?.onError) {
        options.onError(error);
      }
    };

    onStart();
    try {
      const response = await axios<R>({
        method: method,
        url: url,
        data: method === "get" ? {} : data,
        params: method === "get" ? data : {},
      });
      onSuccess(response);
      return Promise.resolve(response);
    } catch (e) {
      if (e instanceof AxiosError) {
        onError(e);
      }
      return Promise.reject(e);
    }
  };

  return reactive({
    fetch,
    hasErrors: computed<boolean>(() => hasErrors.value),
    isFetching: computed<boolean>(() => isFetching.value),
    isFirthFetch: computed<boolean>(() => isFirthFetch.value),
    get: async <R = any>(data?: D, options?: SubmitOptions) => fetch<R>("get", data, options),
    post: async <R = any>(data?: D, options?: SubmitOptions) => fetch<R>("post", data, options),
    put: async <R = any>(data?: D, options?: SubmitOptions) => fetch<R>("put", data, options),
    patch: async <R = any>(data?: D, options?: SubmitOptions) => fetch<R>("patch", data, options),
    delete: async <R = any>(data?: D, options?: SubmitOptions) => fetch<R>("delete", data, options),
  });
}
